import { decryptAES, encryptAES, utf8ToSha256Hex } from "@/utils/aes-utils";
import { convertToObject } from "@/utils/helper";
import { NextResponse } from "next/server";

interface RequestData {
  encdata: string;
  txnPassword: string;
  userId: string;
  errorTag: number;
}

interface EncObj {
  checkSum?: string;
  merchantCode?: string;
  reservationId?: string;
  txnAmount?: string;
  RU?: string;
}

interface ApiResponse {
  message?: string;
  data?: any;
  isSuccess?: boolean;
}

export async function POST(request: Request): Promise<NextResponse> {
  const balanceCheckURL: string = process.env.balanceCheck || "";
  const secretKey: string = process.env.AES_SECRET_KEY || "";
  const MERCODE: string = process.env.MERCHANTCODE || "";
  const iv: string = process.env.AES_IV || "";

  const data: RequestData = await request.json();
  const { encdata } = data;
  //   const postData = decryptAES(encdata, secretKey, iv);
  console.log(encdata, "dv");

  let message: string | null = null;

  let revId: string | undefined = undefined;
  let coId: string | undefined = undefined;
  let txnAmt: number | undefined = 0.0;
  let merCode: string | undefined = undefined;

  let AESENCR: string | null = null;
  let chkSUMSTRING: string | null = null;
  let chkSUMSTRINGFINAL: string | null = null;
  let chkSUM: string | null = null;

  try {
    if (encdata !== null) {
      const dataENC = encdata;
      const DataRet = decryptAES(dataENC, secretKey, iv);
      const encObj: EncObj = convertToObject(DataRet);
      const checksumIndex = DataRet.indexOf("|checkSum=");
      const dataBeforeChecksum =
        checksumIndex !== -1 ? encdata.slice(0, checksumIndex) : encdata;
      const SHA256HX: string = utf8ToSha256Hex(dataBeforeChecksum);

      if (SHA256HX.toUpperCase() === encObj.checkSum) {
        merCode = encObj?.merchantCode;
        revId = encObj?.reservationId;
        coId = encObj?.reservationId;
        txnAmt = Number(encObj?.txnAmount);
        const response = await fetch(
          `${balanceCheckURL}?transactionID=${revId}&corelationID='Verified'`,
          {
            method: "POST",
            headers: {
              "Content-Type": "application/json",
            },
          }
        );
        const responseData: ApiResponse = await response.json();
        console.log(responseData, "d");
        message = responseData?.message || "something went wrong!";
        const isSuccess = responseData?.isSuccess;

        if (isSuccess) {
          if (merCode == MERCODE) {
            chkSUMSTRING = `merchantCode=${merCode}|reservationId=${revId}|txnAmount=${txnAmt}|bankTxnId=${coId}|status=0|statusDesc=Completed successfully`;
            chkSUM = utf8ToSha256Hex(chkSUMSTRING);
            chkSUMSTRINGFINAL = chkSUMSTRING.concat(`|checkSum=${chkSUM}`);
            AESENCR = encryptAES(chkSUMSTRINGFINAL, secretKey, iv);
          } else {
            let retMSG: string | null = null;
            if (merCode != MERCODE) {
              retMSG = `Invalid Merchant Code`;
            } else {
              retMSG = `Invalid Data`;
            }
            chkSUMSTRING = `merchantCode=${merCode}|reservationId=${revId}|txnAmount=${txnAmt}|bankTxnId=${coId}|status=1|${retMSG}`;
            chkSUM = utf8ToSha256Hex(chkSUMSTRING);
            chkSUMSTRINGFINAL = chkSUMSTRING.concat(`|checkSum=${chkSUM}`);
            AESENCR = encryptAES(chkSUMSTRINGFINAL, secretKey, iv);
          }
        } else {
          chkSUMSTRING = `merchantCode=${merCode}|reservationId=${revId}|txnAmount=${txnAmt}|bankTxnId=${coId}|status=1|statusDesc=Invalid Data`;
          chkSUM = utf8ToSha256Hex(chkSUMSTRING);
          chkSUMSTRINGFINAL = chkSUMSTRING.concat(`|checkSum=${chkSUM}`);
          AESENCR = encryptAES(chkSUMSTRINGFINAL, secretKey, iv);
        }
      } else {
        chkSUMSTRING = `merchantCode=${merCode}|reservationId=${revId}|txnAmount=${txnAmt}|bankTxnId=${coId}|status=1|statusDesc=Invalid CheckSum`;
        chkSUM = utf8ToSha256Hex(chkSUMSTRING);
        chkSUMSTRINGFINAL = chkSUMSTRING.concat(`|checkSum=${chkSUM}`);
        AESENCR = encryptAES(chkSUMSTRINGFINAL, secretKey, iv);
      }
    } else {
      chkSUMSTRING = `merchantCode=${merCode}|reservationId=${revId}|txnAmount=${txnAmt}|bankTxnId=${coId}|status=1|statusDesc=Invalid Request Parameter`;
      chkSUM = utf8ToSha256Hex(chkSUMSTRING);
      chkSUMSTRINGFINAL = chkSUMSTRING.concat(`|checkSum=${chkSUM}`);
      AESENCR = encryptAES(chkSUMSTRINGFINAL, secretKey, iv);
    }
  } catch (error) {
    chkSUMSTRING = `merchantCode=${merCode}|reservationId=${revId}|txnAmount=${txnAmt}|bankTxnId=${coId}|status=1|statusDesc=System Error`;
    chkSUM = utf8ToSha256Hex(chkSUMSTRING);
    chkSUMSTRINGFINAL = chkSUMSTRING.concat(`|checkSum=${chkSUM}`);
    AESENCR = encryptAES(chkSUMSTRINGFINAL, secretKey, iv);
  }
  return NextResponse.json({
    encData: AESENCR,
    message: "",
    success: true,
    status: 200,
  });
}
