import { decryptAES, encryptAES, utf8ToSha256Hex } from "@/utils/aes-utils";
import { convertToObject } from "@/utils/helper";
import { NextResponse } from "next/server";

interface RequestData {
  encdata: string;
  txnPassword: string;
  userId: string;
  errorTag: number;
}

interface EncObj {
  checkSum?: string;
  merchantCode?: string;
  reservationId?: string;
  txnAmount?: string;
  RU?: string;
}

interface ApiResponse {
  message?: string;
  data?: any;
  isSuccess?: boolean;
}

export async function POST(request: Request): Promise<NextResponse> {
  const data: RequestData = await request.json();
  const { encdata, txnPassword, userId, errorTag } = data;
  const balanceCheckURL: string = process.env.balanceCheck || "";
  const secretKey: string = process.env.AES_SECRET_KEY || "";
  const merchant: string = process.env.MERCHANTCODE || "";
  const key1: string = process.env.MYKEY1 || "";
  const iv1: string = process.env.MYIV1 || "";
  const iv: string = process.env.AES_IV || "";
  let message: string | null = null;
  if (errorTag == 0) {
    let AESEC: string | null = null;
    try {
      const postData = decryptAES(encdata, secretKey, iv);
      const encObj: EncObj = convertToObject(postData);
      //convert to object that postData

      if (encObj?.merchantCode === merchant) {
        const checksumIndex = postData.indexOf("|checkSum=");
        const dataBeforeChecksum =
          checksumIndex !== -1 ? postData.slice(0, checksumIndex) : postData;

        const SHA256HX: string = utf8ToSha256Hex(dataBeforeChecksum);

        if (SHA256HX.toUpperCase() === encObj.checkSum) {
          const encryptEncdata = encryptAES(postData, key1, iv1);
          const response = await fetch(
            `${balanceCheckURL}?RAIL_ID=${userId}&PWD=${txnPassword}&encData=${encryptEncdata}`,
            {
              method: "POST",
              headers: {
                "Content-Type": "application/json",
              },
            }
          );
          const responseData: ApiResponse = await response.json();
          console.log(responseData, "d");
          message = responseData?.message || "something went wrong!";
          const isSuccess = responseData?.isSuccess;

          if (isSuccess) {
            let chkSUMSTRING = `merchantCode=${encObj.merchantCode}|reservationId=${encObj.reservationId}|txnAmount=${encObj.txnAmount}|bankTxnId=${encObj.reservationId}|status=0|statusDesc=Completed successfully`;
            const chkSUM = utf8ToSha256Hex(chkSUMSTRING);
            chkSUMSTRING = chkSUMSTRING.concat(`|checkSum=${chkSUM}`);
            AESEC = encryptAES(chkSUMSTRING, secretKey, iv);
            console.log(encObj.RU, "l");

            return NextResponse.json({
              message: message ?? "Transaction is Successful",
              RU: encObj.RU,
              encData: AESEC,
              success: true,
              status: 200,
            });
          } else {
            return NextResponse.json({
              errorTag: 0,
              message: message ?? "Invalid credential",
              success: false,
              status: 404,
            });
          }
        } else {
          return NextResponse.json({
            errorTag: 0,
            message:
              message ?? "Invalid Checksum. please Contact Administrator.",
            success: false,
            status: 404,
          });
        }
      } else {
        return NextResponse.json({
          errorTag: 0,
          message: "Invalid Merchant Code. Please Contact Administrator.",
          success: false,
          status: 401,
        });
      }
    } catch (error) {
      return NextResponse.json({
        errorTag: 0,
        message: error || "something went wrong!",
        success: false,
        status: 404,
      });
    }
  } else {
    return NextResponse.json({
      errorTag: 1,
      message: "",
      success: false,
      status: 404,
    });
  }
}
