import moment from "moment";
import { NextRequest, NextResponse } from "next/server";
import { decryptAES } from "./utils/aes-utils";
import { convertToObject } from "./utils/helper";
import process from "process";

interface PaymentData {
  reservationId?: string;
  txnDate?: string;
  txnAmount?: string;
}

export async function middleware(request: NextRequest): Promise<NextResponse> {
  if (request.method === "POST") {
    const secretKey: string = process.env.AES_SECRET_KEY || "";
    const iv: string = process.env.AES_IV || "";
    const data = await request.formData();
    const encdata = data.get("encdata")?.toString();

    if (encdata) {
      try {
        const decryptedData = decryptAES(encdata, secretKey, iv);
        const paymentData: PaymentData = convertToObject(decryptedData);

        const apiResponse = await fetch(new URL("/api/get-html", request.url), {
          method: "POST",
          headers: {
            Authorization: `${process.env.API_SECRET_KEY}`,
          },
        });

        if (apiResponse.ok) {
          let htmlContent = await apiResponse.text();
          htmlContent = htmlContent
            .replace("[reservationId]", paymentData.reservationId || "")
            .replace(
              "[txnDate]",
              moment(paymentData.txnDate, "YYYYMMDD").format("YYYY-MM-DD")
            )
            .replace("[txnAmount]", paymentData.txnAmount || "");

          const response = new NextResponse(htmlContent, {
            headers: { "Content-Type": "text/html" },
          });

          response.cookies.set("encdata", JSON.stringify(encdata));
          return response;
        } else {
          return NextResponse.redirect(new URL("/error", request.url));
        }
      } catch (error) {
        console.error("Error processing middleware:", error);
        return NextResponse.redirect(new URL("/error", request.url));
      }
    } else {
      return NextResponse.redirect(new URL("/error", request.url));
    }
  } else if (request.method === "GET") {
    return NextResponse.redirect(new URL("/error", request.url));
  } else {
    return NextResponse.next();
  }
}

export const config = {
  matcher: "/",
};
